<?php

require_once('api/Simpla.php');
require("utils.php");

class Payture extends Simpla {

    private $settings;

    private $defaults = array(
        'payture_host' => 'https://sandbox.payture.com',
        'payture_method_name' => 'apim/Init',
        'payture_session_type' => 'Pay',
        'payture_taxation_system' => 0,
        'payture_tax' => 6,
        'cheque_message' => 'Payture Cheque',
        'payture_secret_password' => '123'
    );

    private $message = array(
        'RUS' => array(
            'ORDER' => 'Заказ',
            'PAY' => 'Оплатить с Payture',
            'DELIVERY' => 'Доставка',
            'INIT_FAILED' => 'Ошибка! Не удалось инициализировать сессию оплаты',
        ),
        'ENG' => array(
            'ORDER' => 'Order',
            'PAY' => 'Pay with Payture',
            'DELIVERY' => 'Delivery',
            'INIT_FAILED' => 'Error! Failed to initialize payment session',
        )
    );
    
    
    public function checkout_form($order_id, $button_text = null) {
        Logger::info('Initialized checkout with Payture');

        if(empty($button_text))
            $button_text = 'Оплатить с Payture';
            
        $order = $this->orders->get_order((int)$order_id);
        $payment_method = $this->payment->get_payment_method($order->payment_method_id);
        $this->settings = $this->payment->get_payment_settings($payment_method->id);

        $session_id = $this->initialize_session($payment_method, $order);

        if ( empty($session_id) ) {
            return $this->get_error_banner('INIT_FAILED');
        }

        $host = !empty($this->settings['payture_gate_host']) ? $this->settings['payture_gate_host'] : $this->defaults['host'];
        $pay_link = $host.'/apim/Pay?SessionId=' . $session_id;

        return "<div>
            <button class='payture-pay-btn' onclick='window.top.location.href=\"" . $pay_link . "\";'>" .  $this->message("PAY") . "</button>
        </div>";
    }

    private function build_cheque($order) {
        Logger::info("Cheque building started");

        $purchases = $this->orders->get_purchases(array('order_id'=>intval($order->id)));
        if (!$purchases) {
            Logger::error("Error occured while cheque creating");
            return "";
        }

        $positions = array();
        $tax = array_key_exists('payture_tax', $this->settings) ? $this->settings['payture_tax'] : $this->defaults['payture_tax'];

        foreach ($purchases as $purchase) {
            $product_name = htmlspecialchars(addslashes($purchase->product_name));
            $positions[] = array(
                "Quantity" => floatval($purchase->amount),
                "Price" => round($this->money->convert($purchase->price, $payment_method->currency_id, false), 2),
                "Tax" => intval($tax),
                "Text" => $product_name,
            );
        }

        $delivery = $this->delivery->get_delivery($order->delivery_id);
        if ( $delivery->enabled == 1 && !$delivery->separate_payment ) {
            $text = htmlspecialchars(addslashes($this->message('DELIVERY')));
            $positions[] = array(
                "Quantity" => 1,
                "Price" => round($this->money->convert($order->delivery_price, $payment_method->currency_id, false), 2),
                "Tax" => intval($tax),
                "Text" => $text,
            );
        }

        $taxation_system = array_key_exists('payture_taxation_system', $this->settings) ? $this->settings['payture_taxation_system'] : $this->defaults['payture_taxation_system'];
        $cheque_message = array_key_exists('cheque_message', $this->settings) && !empty($this->settings['cheque_message']) ? $this->settings['cheque_message'] : $this->defaults['cheque_message'];
        $email = !empty($order->email) ? $order->email : $this->settings['default_cheque_email'];
        
        $cheque = array(
            "Positions" => $positions,           
            "CheckClose" => array(               
                "TaxationSystem" => intval($taxation_system)			
            ),
			"Message" => $cheque_message,
            "CustomerContact" => $email
        );

        Logger::info("Cheque created [JSON]: " . json_encode($cheque));

        $cheque = base64_encode(json_encode($cheque));
        return $cheque;
    }

    private function message($key) {
        return $this->message[$this->settings['payture_language']][$key];
    }

    private function get_error_banner($message_key) {
        return "<div style='border: 1px solid red; padding: 15px; background-color: #ffd5d5;'>
            <strong>" . $this->message($message_key) . "</strong>
        </div>";
    }

    private function calculate_check_sum($order) {
        $secret_key = !empty($this->settings['payture_secret_password']) ? $this->settings['payture_secret_password'] : $this->defaults['payture_secret_password'];
       
        $purchases = $this->orders->get_purchases(array('order_id'=>intval($order->id)));
        $positions = array();
        foreach ($purchases as $purchase)  {
            $product_name = htmlspecialchars(addslashes($purchase->product_name));
            $positions[] = array(
                "Quantity" => floatval($purchase->amount),
                "Price" => floatval($purchase->price),
                "Text" => $product_name,
            );
        }

        $bundle = array(
            'OrderNumber' => $order->id,
            'Secret' => $secret_key,
            'TotalAmount' => round($this->money->convert($order->total_price, $payment_method->currency_id, false), 2) * 100,
            'Products' => json_encode($positions)
        );
        $sum = join(';', array_map(function($key, $value) { return $key . '=' . $value; }, array_keys($bundle), array_values($bundle)));
        return sha1($sum);
    }

    private function initialize_session($payment_method, $order) {
        Logger::info('Trying to initialize Payture session');

        if (empty($this->settings['payture_terminal_key'])) {
            Logger::error('Empty terminal key');
            die();
        }

        $host = !empty($this->settings['payture_gate_host']) ? $this->settings['payture_gate_host'] : $this->defaults['host'];

        $data_params = array(
            'SessionType' => !empty($this->settings['payture_session_type']) ? $this->settings['payture_session_type'] : $this->defaults['payture_session_type'],
            'OrderId' => 'Order-' . $order->id . time(),
            'Product' => $this->message("ORDER").' #' . $order->id,
            'Amount' => round($this->money->convert($order->total_price, $payment_method->currency_id, false), 2) * 100,
            'Url' => $this->settings['payture_final_url'],
            'IP' => $_SERVER['REMOTE_ADDR'],
            'Total' => round($this->money->convert($order->total_price, $payment_method->currency_id, false), 2),
            'OrderNumber' => $order->id,
            'CheckSum' => $this->calculate_check_sum($order)
        );

        $init_data = join(';', array_map(function($key, $value) { return $key . '=' . $value; }, array_keys($data_params), array_values($data_params)));

        if ($this->settings['payture_cheque'] == 1) {
            $init_data .= ';Cheque=' . $this->build_cheque($order); 
        }

        $init_data = urlencode($init_data);
        $request_params = 'Key='. $this->settings['payture_terminal_key'] . '&Data=' . $init_data;        

        $response_obj = request($host, $this->defaults['payture_method_name'], $request_params);

        if (strtolower($response_obj['Success']) == 'false') {
            Logger::error("Failed to initialize Payture session. Error code: " . $response_obj['ErrCode']);
            return "";
        }

        if ($response_obj['SessionId'] === false) {
            Logger::error("Failed to initialize Payture session. Session value is not found in gate response");
            return "";
        } else {
            return $response_obj['SessionId'];
        }
    }


}

?>